Added new option to interface.conf: "nonotify" to disable notify messages.
[lcr.git] / mISDN.h
diff --git a/mISDN.h b/mISDN.h
index db1a8cb..cbebcac 100644 (file)
--- a/mISDN.h
+++ b/mISDN.h
@@ -9,57 +9,67 @@
 **                                                                           **
 \*****************************************************************************/ 
 
-enum {
-       B_STATE_IDLE,
-       B_STATE_ACTIVATING,
-       B_STATE_ACTIVE,
-       B_STATE_DEACTIVATING,
-};
-
-enum {
-       B_EVENT_ACTIVATE,
-       B_EVENT_ACTIVATED,
-       B_EVENT_DEACTIVATE,
-       B_EVENT_DEACTIVATED,
-};
-
 #define FROMUP_BUFFER_SIZE 1024
 #define FROMUP_BUFFER_MASK 1023
 
 extern int entity;
 extern int mISDNdevice;
 
+extern int mISDNsocket;
+
+enum {
+       B_EVENT_USE,            /* activate/export bchannel */
+       B_EVENT_EXPORTREQUEST,  /* remote app requests bchannel */
+       B_EVENT_IMPORTREQUEST,  /* remote app releases bchannel */
+       B_EVENT_ACTIVATED,      /* DL_ESTABLISH received */
+       B_EVENT_DROP,           /* deactivate/re-import bchannel */
+       B_EVENT_DEACTIVATED,    /* DL_RELEASE received */
+       B_EVENT_EXPORTED,       /* BCHANNEL_ASSIGN received */
+       B_EVENT_IMPORTED,       /* BCHANNEL_REMOVE received */
+       B_EVENT_TIMEOUT,        /* timeout for bchannel state */
+};
+
 /* mISDN port structure list */
 struct mISDNport {
-       net_stack_t nst; /* MUST be the first entry, so &nst equals &mISDNlist */
-       manager_t mgr;
+       struct mlayer3 *ml3;
        struct mISDNport *next;
+       char name[64]; /* name of port, if available */
        struct interface_port *ifport; /* link to interface_port */
 //     int iftype; /* IF_* */
 //     int multilink; /* if set, this port will not support callwaiting */
-       int portnum; /* port number */
+       int portnum; /* port number 1..n */
        int ptp; /* if ptp is set, we keep track of l2link */
        int l1link; /* if l1 is available (only works with nt-mode) */
        int l2link; /* if l2 is available (at PTP we take this serious) */
-//     time_t l1timeout; /* timout when establishing link */
+       unsigned char l2mask[16]; /* 128 bits for each tei */
+       int l1hold; /* set, if layer 1 should be holt */
+       int l2hold; /* set, if layer 2 must be hold/checked */
        time_t l2establish; /* time until establishing after link failure */
        int use; /* counts the number of port that uses this port */
-       int ntmode; /* is TRUE if port is nt mode */
+       int ntmode; /* is TRUE if port is NT mode */
+       int tespecial; /* is TRUE if port uses special TE mode */
        int pri; /* is TRUE if port is a primary rate interface */
        int tones; /* TRUE if tones are sent outside connect state */
        int earlyb; /* TRUE if tones are received outside connect state */
-       int upper_id; /* id to transfer data down */
-       int lower_id; /* id to transfer data up */
-       int d_stid;
        int b_num; /* number of bchannels */
        int b_reserved; /* number of bchannels reserved or in use */
-       class PmISDN *b_port[128]; /* maximum number of ports shall be 128 due to S0 / E1 / special E1 */
-       int b_stid[128];
-       int b_addr[128];
-       int b_state[128]; /* state 0 = IDLE */
-       int procids[128]; /* keep track of free ids */
+       class PmISDN *b_port[128]; /* bchannel assigned to port object */
+       struct mqueue upqueue;
+       int b_socket[128];
+       int b_mode[128]; /* B_MODE_* */
+       int b_state[128]; /* statemachine, 0 = IDLE */
+       double b_timer[128]; /* timer for state machine */
+       int b_remote_id[128]; /* the socket currently exported (0=none) */
+       unsigned int b_remote_ref[128]; /* the ref currently exported */
        int locally; /* local causes are sent as local causes not remote */
-       msg_queue_t downqueue;          /* l4->l3 */
+       int los, ais, rdi, slip_rx, slip_tx;
+
+       /* gsm */
+       int gsm; /* this is the (only) GSM interface */
+       int lcr_sock; /* socket of loopback on LCR side */
+
+       /* ss5 */
+       unsigned int ss5; /* set, if SS5 signalling enabled, also holds feature bits */
 };
 extern mISDNport *mISDNport_first;
 
@@ -81,40 +91,41 @@ calls with no bchannel (call waiting, call on hold).
 /* mISDN none-object functions */
 int mISDN_initialize(void);
 void mISDN_deinitialize(void);
-void mISDN_port_info(void);
-struct mISDNport *mISDNport_open(int port, int ptp, int ptmp, struct interface *interface);
+int mISDN_getportbyname(int sock, int cnt, char *portname);
+struct mISDNport *mISDNport_open(struct interface_port *ifport);
+void mISDNport_static(struct mISDNport *mISDNport);
 void mISDNport_close_all(void);
 void mISDNport_close(struct mISDNport *mISDNport);
 void mISDN_port_reorder(void);
 int mISDN_handler(void);
-void enc_ie_cause_standalone(unsigned char **ntmode, msg_t *msg, int location, int cause);
-void ph_control(struct mISDNport *mISDNport, class PmISDN *isdnport, unsigned long b_addr, int c1, int c2, char *trace_name, int trace_value);
-void ph_control_block(struct mISDNport *mISDNport, unsigned long b_addr, int c1, void *c2, int c2_len, char *trace_name, int trace_value);
-msg_t *create_l2msg(int prim, int dinfo, int size);
-void setup_queue(struct mISDNport *mISDNport, int link);
-int stack2manager_nt(void *dat, void *arg);
-int stack2manager_te(struct mISDNport *mISDNport, msg_t *msg);
-void chan_trace_header(struct mISDNport *mISDNport, class PmISDN *port, char *msgtext, int direction);
-void l1l2l3_trace_header(struct mISDNport *mISDNport, class PmISDN *port, unsigned long prim, int direction);
+void enc_ie_cause_standalone(struct l3_msg *l3m, int location, int cause);
+int stack2manager(struct mISDNport *mISDNport, unsigned int cmd, unsigned int pid, struct l3_msg *l3m);
+void ph_control(struct mISDNport *mISDNport, class PmISDN *isdnport, unsigned int handle, unsigned int c1, unsigned int c2, const char *trace_name, int trace_value);
+void ph_control_block(struct mISDNport *mISDNport, unsigned int handle, unsigned int c1, void *c2, int c2_len, const char *trace_name, int trace_value);
+void chan_trace_header(struct mISDNport *mISDNport, class PmISDN *port, const char *msgtext, int direction);
+void l1l2l3_trace_header(struct mISDNport *mISDNport, class PmISDN *port, unsigned int prim, int direction);
 void bchannel_event(struct mISDNport *mISDNport, int i, int event);
+void message_bchannel_from_remote(class JoinRemote *joinremote, int type, unsigned int handle);
 
 
 /* mISDN port classes */
 class PmISDN : public Port
 {
        public:
-       PmISDN(int type, struct mISDNport *mISDNport, char *portname, struct port_settings *settings, int channel, int exclusive);
+       PmISDN(int type, struct mISDNport *mISDNport, char *portname, struct port_settings *settings, int channel, int exclusive, int mode);
        ~PmISDN();
-       void bchannel_receive(iframe_t *frm);
+       void bchannel_receive(struct mISDNhead *hh, unsigned char *data, int len);
        int handler(void);
        void transmit(unsigned char *buffer, int length);
-       int message_epoint(unsigned long epoint_id, int message, union parameter *param);
-       void message_mISDNsignal(unsigned long epoint_id, int message_id, union parameter *param);
-       void message_crypt(unsigned long epoint_id, int message_id, union parameter *param);
+       int message_epoint(unsigned int epoint_id, int message, union parameter *param);
+       void message_mISDNsignal(unsigned int epoint_id, int message_id, union parameter *param);
+       void message_crypt(unsigned int epoint_id, int message_id, union parameter *param);
        struct mISDNport *p_m_mISDNport;        /* pointer to port */
        int p_m_delay;                          /* use delay instead of dejitter */
-       int p_m_txvol, p_m_rxvol;               /* volume shift (0 = no change) */
+       int p_m_tx_gain, p_m_rx_gain;           /* volume shift (0 = no change) */
+       char p_m_pipeline[256];                 /* filter pipeline */
        int p_m_echo, p_m_conf;                 /* remote echo, conference number */
+       int p_m_mute;                           /* if set, conf is disconnected */
        int p_m_tone;                           /* current kernel space tone */
        int p_m_rxoff;                          /* rx from driver is disabled */
 //     int p_m_nodata;                         /* all parties within a conf are isdn ports, so pure bridging is possible */
@@ -123,8 +134,8 @@ class PmISDN : public Port
        int p_m_joindata;                       /* the call requires data due to no briging capability */
 
        int p_m_load;                           /* current data in dsp tx buffer */
-       unsigned long p_m_last_tv_sec;          /* time stamp of last handler call, (to sync audio data */
-       unsigned long p_m_last_tv_msec;
+       unsigned int p_m_last_tv_sec;           /* time stamp of last handler call, (to sync audio data */
+       unsigned int p_m_last_tv_msec;
 //     int p_m_fromup_buffer_readp;            /* buffer for audio from remote endpoint */
 //     int p_m_fromup_buffer_writep;
 //     unsigned char p_m_fromup_buffer[FROMUP_BUFFER_SIZE];
@@ -141,11 +152,12 @@ class PmISDN : public Port
        int p_m_crypt_listen_state;
        int p_m_crypt_listen_len;
        unsigned char p_m_crypt_listen_msg[1100];
-       unsigned long p_m_crypt_listen_crc;
+       unsigned int p_m_crypt_listen_crc;
        void cryptman_listen_bch(unsigned char *p, int l);
 
-       void set_tone(char *dir, char *name);
+       void set_tone(const char *dir, const char *name);
        void set_echotest(int echotest);
+       void set_conf(int oldconf, int newconf);
 
        int p_m_portnum;                        /* used port number (1...n) */
        int p_m_b_index;                        /* index 0,1 0..29 */
@@ -154,10 +166,25 @@ class PmISDN : public Port
        int p_m_b_reserve;                      /* set if channel is reserved */
 //     long long p_m_jittercheck;              /* time of audio data */
 //     long long p_m_jitterdropped;            /* number of bytes dropped */
+       int p_m_b_mode;                         /* bchannel mode */
        int p_m_delete;                         /* true if obj. must del. */
        int p_m_hold;                           /* if port is on hold */
-       unsigned long p_m_timeout;              /* timeout of timers */
+       unsigned int p_m_timeout;               /* timeout of timers */
        time_t p_m_timer;                       /* start of timer */
+       unsigned int p_m_remote_ref;            /* join to export bchannel to */
+       int p_m_remote_id;                      /* sock to export bchannel to */
+
+       int p_m_inband_send_on;                 /* triggers optional send function */
+       int p_m_inband_receive_on;              /* triggers optional receive function */
+       int p_m_mute_on;                        /* if mute is on, bridge is removed */
+       virtual int inband_send(unsigned char *buffer, int len);
+       void inband_send_on(void);
+       void inband_send_off(void);
+       virtual void inband_receive(unsigned char *buffer, int len);
+       void inband_receive_on(void);
+       void inband_receive_off(void);
+       void mute_on(void);
+       void mute_off(void);
 
        int seize_bchannel(int channel, int exclusive); /* requests / reserves / links bchannels, but does not open it! */
        void drop_bchannel(void);